﻿using FarPoint.Win.Spread;
using FarPoint.Win.Spread.CellType;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Text;

namespace AnimateCellType
{
  /// <summary>
  /// Sample animate celltype
  /// </summary>
  class AnimateCellType : ImageCellType, IDisposable
  {
    #region Variables and Initializing
    Image currentImage = null;
    private SheetView sheet = null;
    private int rowIndex = -1;
    private int columnIndex = -1;

    /// <summary>
    /// Constructor
    /// </summary>
    /// <param name="owner">Need to notify invalidate, check and control Spread from celltype</param>
    /// <param name="row">row index</param>
    /// <param name="column">column index</param>
    public AnimateCellType(SheetView owner, int row, int column)
    {
      if (owner == null)
        throw new ArgumentNullException("owner");
      if (owner.FpSpread == null)
        throw new ArgumentNullException("owner.FpSpread");
      if (row < 0 || row >= owner.RowCount)
        throw new ArgumentOutOfRangeException("row");
      if (column < 0 || column >= owner.ColumnCount)
        throw new ArgumentOutOfRangeException("column");
      sheet = owner;
      rowIndex = row;
      columnIndex = column;
      sheet.Disposing += sheet_Disposing;
      sheet.CellChanged += sheet_CellChanged;
      sheet.FpSpread.ActiveSheetChanged += FpSpread_ActiveSheetChanged;
      sheet.FpSpread.Disposed += FpSpread_Disposed;
    }
    #endregion

    #region Dispose
    void FpSpread_Disposed(object sender, EventArgs e)
    {
      StopAnimate();
      sheet.FpSpread.Disposed -= FpSpread_Disposed;
      sheet.FpSpread.ActiveSheetChanged -= FpSpread_ActiveSheetChanged;
    }
    void sheet_Disposing(object sender, EventArgs e)
    {
      StopAnimate();
      sheet.Disposing -= sheet_Disposing;
      sheet.CellChanged -= sheet_CellChanged;
    }
    public void Dispose()
    {
      StopAnimate();
      sheet.FpSpread.Disposed -= FpSpread_Disposed;
      sheet.FpSpread.ActiveSheetChanged -= FpSpread_ActiveSheetChanged;
      sheet.Disposing -= sheet_Disposing;
      sheet.CellChanged -= sheet_CellChanged;
    }
    #endregion

    /// <summary>
    /// Should not animate when cell is not display
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    void FpSpread_ActiveSheetChanged(object sender, EventArgs e)
    {
      if (sheet.FpSpread.ActiveSheet != sheet)
        StopAnimate();
    }

    /// <summary>
    /// Should update in case value changed
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    void sheet_CellChanged(object sender, SheetViewEventArgs e)
    {
      if (e.Row == rowIndex && e.Column == columnIndex)
        StopAnimate();
    }

    /// <summary>
    /// Stop Animate
    /// </summary>
    void StopAnimate()
    {
      if (currentImage != null)
        lock (currentImage)
        {
          ImageAnimator.StopAnimate(currentImage, new EventHandler(OnFrameChanged));
          currentImage.Dispose();
          currentImage = null;
        }
    }

    /// <summary>
    /// Start Animate
    /// </summary>
    /// <param name="value"></param>
    void StartAnimate(object value)
    {
      StopAnimate();
      Image imgValue = ImageCellType.GetImage(value);
      if (imgValue != null && ImageAnimator.CanAnimate(imgValue))
      {
        currentImage = imgValue.Clone() as Image;
        ImageAnimator.Animate(currentImage, new EventHandler(OnFrameChanged));
      }
    }

    /// <summary>
    /// Original cell Paint or repaint cause by image frame per second rate
    /// </summary>
    /// <param name="g"></param>
    /// <param name="r"></param>
    /// <param name="appearance"></param>
    /// <param name="value"></param>
    /// <param name="isSelected"></param>
    /// <param name="isLocked"></param>
    /// <param name="zoomFactor"></param>
    public override void PaintCell(Graphics g, Rectangle r, FarPoint.Win.Spread.Appearance appearance, object value, bool isSelected, bool isLocked, float zoomFactor)
    {
      if (currentImage == null)
      {
        base.PaintCell(g, r, appearance, value, isSelected, isLocked, zoomFactor);
       StartAnimate(value);
      }
      else lock (currentImage)
          base.PaintCell(g, r, appearance, currentImage, isSelected, isLocked, zoomFactor);
    }

    /// <summary>
    /// depend on frame per second rate, it will notify to update frame for painting
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    void OnFrameChanged(object sender, EventArgs e)
    {
      lock (currentImage)
        ImageAnimator.UpdateFrames(currentImage);
      int rv = sheet.FpSpread.GetRowViewportCount();
      int cv = sheet.FpSpread.GetColumnViewportCount();
      Rectangle spreadRect = sheet.FpSpread.Bounds;
      for (int r = -1; r < rv; r++)
        for (int c = -1; c < cv; c++)
        {
          Rectangle rect = sheet.FpSpread.GetCellRectangle(r, c, rowIndex, columnIndex);
          rect.Intersect(spreadRect);
          if (!rect.IsEmpty)
            sheet.FpSpread.Invalidate();
        }
    }
  }
}
